import React from 'react';
import PropTypes from 'prop-types';
import Measure from 'react-measure';
import { Transition, animated, interpolate } from 'react-spring';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var styles = {
  outer: {
    position: 'relative',
    width: '100%',
    height: '100%'
  },
  inner: {
    position: 'relative',
    width: '100%',
    overflow: 'hidden',
    minHeight: '100%'
  },
  cell: {
    position: 'absolute',
    willChange: 'transform, width, height, opacity'
  }
};

var Grid =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Grid, _React$Component);

  function Grid() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.state = {
      width: 0,
      height: 0,
      open: undefined,
      lastOpen: undefined
    };

    _this.scrollOut = function (e) {
      if (!_this.props.lockScroll) {
        _this.state.open && _this.toggle(undefined);
        _this.clicked = false;
      }
    };

    _this.toggle = function (key) {
      return _this.setState(function (state) {
        return {
          lastOpen: state.open,
          open: state.open === key ? undefined : key
        };
      }, function () {
        return _this.clicked = true;
      });
    };

    _this.resize = function (width, height, props) {
      var _this$setState;

      return _this.setState((_this$setState = {}, _this$setState[width] = props.client.width, _this$setState[height] = props.client.height, _this$setState));
    };

    _this.resizeOuter = function (props) {
      return _this.resize('widthOuter', 'heightOuter', props);
    };

    _this.resizeInner = function (props) {
      return _this.resize('width', 'height', props);
    };

    _this.update = function (_ref) {
      var key = _ref.key,
          x = _ref.x,
          y = _ref.y,
          width = _ref.width,
          height = _ref.height;
      var open = _this.state.open === key;
      return {
        opacity: _this.state.open && !open ? 0 : 1,
        x: open ? _this.outerRef.scrollLeft : x,
        y: open ? _this.outerRef.scrollTop : y,
        width: open ? _this.state.width : width,
        height: open ? _this.state.heightOuter : height
      };
    };

    return _this;
  }

  var _proto = Grid.prototype;

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.clicked = false;
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props = this.props,
        children = _this$props.children,
        columns = _this$props.columns,
        margin = _this$props.margin,
        occupySpace = _this$props.occupySpace,
        impl = _this$props.impl,
        config = _this$props.config,
        data = _this$props.data,
        keys = _this$props.keys,
        heights = _this$props.heights,
        closeDelay = _this$props.closeDelay,
        lockScroll = _this$props.lockScroll,
        rest = _objectWithoutPropertiesLoose(_this$props, ["children", "columns", "margin", "occupySpace", "impl", "config", "data", "keys", "heights", "closeDelay", "lockScroll"]);

    var _this$state = this.state,
        lastOpen = _this$state.lastOpen,
        open = _this$state.open,
        width = _this$state.width;
    var column = 0;
    var columnHeights = new Array(columns).fill(0);
    var displayData = data.map(function (child, i) {
      var index = occupySpace ? columnHeights.indexOf(Math.min.apply(Math, columnHeights)) : column++ % columns;
      var cellWidth = width / columns - margin / (1 - 1 / (columns + 1));
      var left = cellWidth * index;
      var offset = (index + 1) * margin;
      var top = columnHeights[index] + margin;
      var height = typeof heights === 'function' ? heights(child) : heights;
      columnHeights[index] += height + margin;
      return {
        x: margin ? left + offset : left,
        y: top,
        width: cellWidth,
        height: height,
        key: keys(child),
        object: child
      };
    });
    var overflow = lockScroll ? open ? 'hidden' : 'auto' : 'auto';
    var height = Math.max.apply(Math, columnHeights) + margin;
    return React.createElement(Measure, {
      client: true,
      innerRef: function innerRef(r) {
        return _this2.outerRef = r;
      },
      onResize: this.resizeOuter
    }, function (_ref2) {
      var measureRef = _ref2.measureRef;
      return React.createElement("div", _extends({
        ref: measureRef,
        style: _extends({}, styles.outer, _this2.props.style, {
          overflow: overflow
        })
      }, rest, {
        onScroll: _this2.scrollOut,
        onWheel: _this2.scrollOut,
        onTouchMove: _this2.scrollOut
      }), React.createElement(Measure, {
        client: true,
        innerRef: function innerRef(r) {
          return _this2.innerRef = r;
        },
        onResize: _this2.resizeInner
      }, function (_ref3) {
        var measureRef = _ref3.measureRef;
        return React.createElement("div", {
          ref: measureRef,
          style: _extends({}, styles.inner, {
            height: height
          })
        }, React.createElement(Transition, {
          native: true,
          delay: _this2.clicked && !open ? closeDelay : 0,
          items: displayData,
          keys: function keys(d) {
            return d.key;
          },
          from: {
            opacity: 0
          },
          leave: {
            opacity: 0
          },
          enter: _this2.update,
          update: _this2.update,
          impl: impl,
          config: config
        }, displayData.map(function (c, i) {
          return function (_ref4) {
            var opacity = _ref4.opacity,
                x = _ref4.x,
                y = _ref4.y,
                width = _ref4.width,
                height = _ref4.height;
            return React.createElement(animated.div, {
              style: _extends({}, styles.cell, {
                opacity: opacity,
                width: width,
                height: height,
                zIndex: lastOpen === c.key || open === c.key ? 1000 : i,
                transform: interpolate([x, y], function (x, y) {
                  return "translate3d(" + x + "px," + y + "px, 0)";
                })
              }),
              children: children(c.object, open === c.key, function () {
                return _this2.toggle(c.key);
              })
            });
          };
        })));
      }));
    });
  };

  return Grid;
}(React.Component);

Grid.defaultProps = {
  occupySpace: true,
  columns: 3,
  margin: 0,
  heights: 400,
  lockScroll: false,
  closeDelay: 0
};
process.env.NODE_ENV !== "production" ? Grid.propTypes = {
  data: PropTypes.array,
  keys: PropTypes.func,
  occupySpace: PropTypes.bool,
  columns: PropTypes.number,
  margin: PropTypes.number,
  heights:
  /*#__PURE__*/
  PropTypes.oneOfType([PropTypes.func, PropTypes.number]),
  lockScroll: PropTypes.bool,
  closeDelay: PropTypes.number
} : void 0;

export default Grid;
